use askama::Template;
use axum::extract::{Path, State};
use serde_json::json;

use crate::{
    http::response::{reply, HandlerResponse},
    node::container::{get_info, ContainerInfo},
    AppState,
};

#[derive(Template)]
#[template(path = "container/get-one.html")]
struct GetOneTemplate {
    container_name: String,
    info: ContainerInfo,
}

pub(super) async fn get_one(
    Path(container_name): Path<String>,
    State(state): State<AppState>,
) -> HandlerResponse {
    let info = get_info(&state.docker, &container_name).await?;

    reply(
        json!({
            "name": container_name,
            "info": info,
        }),
        GetOneTemplate {
            container_name,
            info,
        },
    )
}
