use anyhow::Result;
use futures_util::future::try_join;
use std::path::{Path, PathBuf};
use tokio::fs;

use crate::git::ThreadSafeRepository;

use super::{arion, COMPOSE_FILE, PACKAGE_CONTENTS, PACKAGE_FILE};

pub async fn create(
    repository: ThreadSafeRepository,
    stack_name: &str,
    source: &str,
) -> Result<()> {
    // Calculate stack directory and create it
    let stack_path = repository.path.join(stack_name);
    fs::create_dir_all(&stack_path).await?;

    // Create package file and compose file
    try_join(
        fs::write(stack_path.join(PACKAGE_FILE), PACKAGE_CONTENTS),
        fs::write(stack_path.join(COMPOSE_FILE), source),
    )
    .await?;

    // Commit everything
    repository.commit_files(
        &[
            &PathBuf::from(format!("{}/{}", stack_name, PACKAGE_FILE)),
            &PathBuf::from(format!("{}/{}", stack_name, COMPOSE_FILE)),
        ],
        format!("Created stack {}", stack_name).as_str(),
    )?;

    Ok(())
}

pub async fn remove(
    base_dir: &Path,
    arion_bin: &Path,
    repository: ThreadSafeRepository,
    stack_name: &str,
) -> Result<()> {
    // Remove all containers and resources
    arion::command(base_dir, stack_name, arion_bin, arion::StackCommand::Down).await?;

    // Remove from repository
    repository.remove_folder(
        &PathBuf::from(stack_name),
        format!("Removed stack {}", stack_name).as_str(),
    )?;

    // Remove from disk
    fs::remove_dir_all(repository.path.join(stack_name)).await?;

    Ok(())
}
